Package: GECal
Type: Package
Title: Generalized Entropy Calibration
Version: 0.1.7
Authors@R: c(person("Yonghyun","Kwon", role=c("aut","cre"), email="yhkwon@kma.ac.kr", comment = c(ORCID = "0000-0001-9923-6790")), 
			person("Jae Kwang","Kim", role="aut", email="jkim@iastate.edu", comment = c(ORCID = "0000-0002-0246-6029")),
      person("Yumou","Qiu", role="aut",email="qiuyumou@math.pku.edu.cn", comment = c(ORCID = "0000-0003-4846-1263"))
			)
Description: Generalized Entropy Calibration produces calibration weights 
    using generalized entropy as the objective function for optimization. 
    This approach, as implemented in the 'GECal' package, is based on 
    Kwon, Kim, and Qiu (2024) <doi:10.48550/arXiv.2404.01076>. 'GECal' incorporates design weights into the constraints to maintain design consistency, rather than including 
    them in the objective function itself.
Encoding: UTF-8
URL: https://github.com/yonghyun-K/GECal
BugReports: https://github.com/yonghyun-K/GECal/issues
Depends: R (>= 2.10.0)
LazyData: true
Imports: nleqslv
Suggests: sampling
RoxygenNote: 7.3.2
License: MIT + file LICENSE
NeedsCompilation: no
Packaged: 2025-07-24 08:24:36 UTC; ghkfk
Author: Yonghyun Kwon [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9923-6790>),
  Jae Kwang Kim [aut] (ORCID: <https://orcid.org/0000-0002-0246-6029>),
  Yumou Qiu [aut] (ORCID: <https://orcid.org/0000-0003-4846-1263>)
Maintainer: Yonghyun Kwon <yhkwon@kma.ac.kr>
Repository: CRAN
Date/Publication: 2025-07-24 08:50:10 UTC
Built: R 4.6.0; ; 2026-04-26 18:48:37 UTC; unix
